/*
 * This demo prints a random alpha-numerical string on
 * a given serial interface. It then expects the user to retype
 * that string on the same interface and shows 'Correct!' or 'Wrong!'
 * on the console depending on the input. 
 * 
 * Usage: serial_demo /dev/ttyAM0 - uses DUART
 *                    /dev/ttySP3 - uses AUART3
 *                    /dev/ttySP0 - uses AUART0
 */

#include "serial_lib.h"
#include <time.h>	/* time */

#define SERIALIF	"/dev/ttySP3"

static char *GenerateRandomWord(int length)
{
	char pool[] = "0123456789abcdefghijklmnopqrstuvwxyz";
	int digit, i = 0;
	char *result;

	result = malloc(length);
	/* take current Epoch time as random number generator seed */
	srand(time(NULL));

	/* fill result with random pool digits */
	while (length > i)
	{
		digit = rand() % strlen(pool);
		result[i++] = pool[digit];
	}

	result[length] = 0;

	return result;
}

int main(int argc, char *argv[])
{
	char *Buffer, *Word;

	printf("- TQ MBa28 Serial demo -\n");
	printf("Use: %s <path/to/tty/interface>\n", argv[0]);
	printf("Without parameter uses /dev/ttySP3 on default.\n\n");

	/* when parameter present, use it as interface path */
	if (argc > 1)
	{
		Buffer = malloc(strlen(argv[1]));
		strncpy(Buffer, argv[1], strlen(argv[1]));
	}
	else
	{
		Buffer = malloc(strlen(SERIALIF));
		strncpy(Buffer, SERIALIF, strlen(SERIALIF));
	}

	if (1 != TTY_Init(Buffer))
	{
		printf("Error initialising interface!\n");
		return -1;
	}

	Word = GenerateRandomWord(5);

	printf("Wrote the word %s on %s.\n", Word, Buffer);
	printf("Please retype it on that interface.\n");

	/* Send word to interface */
	Buffer = realloc(Buffer, 11+strlen(Word));
	snprintf(Buffer, 11+strlen(Word), "\r\nWord: %s\r\n", Word);
	TTY_Write(tty, Buffer, strlen(Buffer));

	/* Send retyping prompt to interface */
	Buffer = realloc(Buffer, 19);
	snprintf(Buffer, 19, "Re-type the word: ");
	TTY_Write(tty, Buffer, strlen(Buffer));

	/* Expect user input */
	Buffer = realloc(Buffer, strlen(Word));
	Buffer = TTY_Read(tty, strlen(Word));

	/* And compare against the generated word */
	if (0 == strncmp(Word, Buffer, strlen(Word)))
	{
		printf("Correct!\n");
	}
	else
	{
		printf("Wrong!\n");
	}

	TTY_Close(tty);
	free(Buffer);
	free(Word);

	return 1;
}

